#!/bin/sh

# main entry point
############################################################

export PATH=$PATH:/sbin
VERSION=50
APPVERSION=0

USRLIBDIR='/usr/lib'
APPDIR='/usr/bin/czsensor'
REFERENCEDIR='/reference'
KERNEL='/lib/modules/3rdparty'
APACHE_DIR='/usr/bin/apache'
SRCDIR='/tmp/czsensorupdate'

J1939LIB='libJ1939.so.1.17'
SERPROTLIB='libserprot.so.1.1'
MCSOEMLIB='libmcsoem.so.1.1'
MOLUPLIB='libmobileolup.so'
USBSPECLIB='libusbspect.so.0.0.0'
CGICCLIB='libcgicc.so.5.0.1'
LOG4CXXLIB='liblog4cxx.so.9.0.2'
EXPATLIB='libexpat.so.0.3.0'
RTLIB='librt-2.3.1.so'
APACHEBIN='apache_image.tar'
DLLCAN='DLLcan.o'
DBLIB='libdb-4.0.so'
APRLIB='libapr.so.0.0.0'
APRUTILLIB='libaprutil.so.0.0.0'

pathToSrc=/mtd/usbstick


# ISSUE JD-1180
# replace busybox tar by gnu tar
replace_tar()
{
	if [ -f $pathToSrc/tar ]; then
		echo
		echo "replacing tar"
		echo "------------------------"
		echo
		rm /bin/tar
		cp $pathToSrc/tar /data/tar
		chmod 775 /data/tar
		ln -s /data/tar /bin/tar
		tar --version | grep tar 
	fi
}

# preparing directories
create_dirs_and_files()
{
	mkdir -p /usr/bin/czsensor/reference

	mkdir -p /data/crop0/ingredient0
	mkdir -p /data/crop0/inactive
	chown http:http /data/crop0/ingredient0
	chown http:http /data/crop0/inactive
	chmod 770 /data/crop0/ingredient0
	chmod 770 /data/crop0/inactive

	# create file and set rights/owner for CSPRNG
	touch /data/csprng
	touch /data/csprng_log
	chown http:http /data/csprng
	chown http:http /data/csprng_log
	chmod 666 /data/csprng
	chmod 666 /data/csprng_log

	# create files for internal sensor status log
	touch /data/errorStateLog
	touch /data/errorStateLog.old
	chmod 666 /data/errorStateLog
	chmod 666 /data/errorStateLog.old

	cd $pathToSrc
}

add_prerun_functionality()
{
	# add prerun functionality
	#####################################
	echo
	echo "checking inittab"
	echo "--------------------------------"
	echo

	AUTOUPDATECMD="::wait:/usr/bin/czsensor/prerun.sh"
	if [ -z $(cat /etc/inittab | grep prerun.sh) ]; then
	   echo "adding line $AUTOUPDATECMD"
	   echo  $AUTOUPDATECMD >> /etc/inittab
	else
	   echo "prerun.sh already enabled"
	fi
	# fix JD-1452
	if [ -f /usr/bin/czsensor/prerun.sh ]; then
	   echo "prerun.sh script already present"   
	else
	   echo "extracting prerun.sh script..."
	   (tar -xvf czsensor.tar    -C /usr/bin/czsensor    prerun.sh 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	   echo "changing permissions..."
	   chmod 755 /usr/bin/czsensor/prerun.sh
	fi
}

extract_tarball()
{
	# extract tarball
	#####################################
	
	echo
	echo "extracting files"
	echo "--------------------------------"
	echo
	(tar -xvf $pathToSrc/czsensor.tar -C /data czsensorupdate.tar.gz 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1

	(tar -xvf $pathToSrc/czsensor.tar -C /data autoupdate.sh 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	chmod 755 /data/autoupdate.sh
}




install_app_binaries()
{
	echo
	echo "installing application binaries"
	echo "--------------------------------"
	echo
	cd /data

	#
	# czsensor application deployment
	#
	(tar -xvf czsensor.tar    -C /usr/bin/czsensor              czsensor.config   2>&1 1>&3 | grep -v "future" 1>&2) 3>&1	
	(tar -xvf czsensor.tar    -C $USRLIBDIR    libcz.0.0.0         2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C /data                          czsensor          2>&1 1>&3 | grep -v "future" 1>&2) 3>&1

	rm /usr/bin/czsensor/czsensor
	ln -sf /data/czsensor /usr/bin/czsensor/czsensor
	chmod a+x /data/czsensor

	#
	# initial references deployment
	#
	(tar -xvf czsensor.tar    -C /usr/bin/czsensor/reference    externaldark.spc  2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C /usr/bin/czsensor/reference    externalwhite.spc 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C /usr/bin/czsensor/reference    internalwhite.spc 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C /usr/bin/czsensor/reference    internaldark.spc  2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C /etc                           inittab           2>&1 1>&3 | grep -v "future" 1>&2) 3>&1	
	
	#
	# WebUI content deployment
	#
	rm -f   $APACHE_DIR/cgi-bin/*	
	rm -f   $APACHE_DIR/htdocs/*
	rm -f   $APACHE_DIR/htdocs/images/*
	rm -f   $APACHE_DIR/htdocs/css/*
	rm -f   $APACHE_DIR/htdocs/js/*
	
	(tar -xvf czsensor.tar    -C $APACHE_DIR/cgi-bin          measurepage                               2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C $APACHE_DIR/cgi-bin          measurementsettingspage                   2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C $APACHE_DIR/cgi-bin          systemsettingspage                        2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C $APACHE_DIR/cgi-bin          systemstatuspage                          2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C $APACHE_DIR/cgi-bin          calibrationadministrationpage             2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C $APACHE_DIR/cgi-bin          qv                                        2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C $APACHE_DIR/cgi-bin          servicesettingspage                       2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C $APACHE_DIR/htdocs/images/   progressBarAjaxLoader.gif                 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C $APACHE_DIR/htdocs/images    DE.png                                    2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C $APACHE_DIR/htdocs/images    ES.png                                    2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C $APACHE_DIR/htdocs/images    FR.png                                    2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C $APACHE_DIR/htdocs/images    US.png                                    2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C $APACHE_DIR/htdocs/css       style.css                                 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C $APACHE_DIR/htdocs/js        CzScripts.js                              2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C $APACHE_DIR/htdocs           calibrationadministrationpageTemplate.htm 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C $APACHE_DIR/htdocs           measurementsettingspageTemplate.htm       2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C $APACHE_DIR/htdocs           measurementsettingspageTemplateManure.htm 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C $APACHE_DIR/htdocs           measurepageTemplate.htm                   2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C $APACHE_DIR/htdocs           servicesettingspageTemplate.htm           2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C $APACHE_DIR/htdocs           systemsettingspageTemplate.htm            2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C $APACHE_DIR/htdocs           systemstatuspageTemplate.htm              2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C $APACHE_DIR/htdocs           systemstatuspageTemplateManure.htm        2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C $APACHE_DIR/htdocs           qvTemplate.htm                            2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C $APACHE_DIR/htdocs           qvTemplateManure.htm                      2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C $APACHE_DIR/htdocs           qvSelfTestTemplate.htm                    2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	chmod -R 644       $APACHE_DIR/htdocs
	chown -R http:http $APACHE_DIR/htdocs
	chmod -R 755       $APACHE_DIR/cgi-bin
	chown -R http:http $APACHE_DIR/cgi-bin
}

# JD-2188 [FR] get calibration information (crop, ingredient) from AMO file header
install_libraries()
{
	echo
	echo "installing libraries"
	echo "--------------------------------"
	echo
	(tar -xvf czsensor.tar    -C $USRLIBDIR $J1939LIB 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C $USRLIBDIR $SERPROTLIB 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C $USRLIBDIR $MCSOEMLIB 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C $USRLIBDIR $MOLUPLIB 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C $USRLIBDIR $USBSPECLIB 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C $USRLIBDIR $CGICCLIB 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C $USRLIBDIR $LOG4CXXLIB 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C $USRLIBDIR $EXPATLIB 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C $USRLIBDIR $APRLIB 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C $USRLIBDIR $APRUTILLIB 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C /lib $RTLIB 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C /lib $DBLIB 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C /sbin ldconfig 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	#ldconfig -l /usr/lib/libcz.0.0.0
	ln -sf /usr/lib/libcz.0.0.0 /usr/lib/libcz.so
	ldconfig -l /usr/lib/libapr.so.0.0.0
	ldconfig -l /usr/lib/libaprutil.so.0.0.0
	ldconfig -n /usr/lib
}


# JD-2188 [FR] get calibration information (crop, ingredient) from AMO file header
install_models()
{
	echo
	echo "installing new / updating old models if necessary"
	echo "----------------------------------"
	echo
	cd /data
# JD-3481 [CR] Update v42 deletes all AMO files
	SW_VERSION=`cat /data/version`
	if [ -z "$SW_VERSION" ] ; then
        echo "Installed Software does not report a version (current SW Version '$SW_VERSION' < V42)."
	elif [ $SW_VERSION -lt 42 ]; then
		echo "Old Sensor Software detected (current SW Version '$SW_VERSION' < V42)."
	elif [ $SW_VERSION -gt 999 ]; then
		echo "Test Sensor Software detected (current SW Version '$SW_VERSION' > V999)."
	fi

	if [ -z "$SW_VERSION" ]  ||  [ $SW_VERSION -lt 42 ]  ||  [ $SW_VERSION -gt 999 ]; then
		echo "--- Removing old model files..."
		rm -f /data/crop0/ingredient0/*.AMO
		rm -f /data/crop0/ingredient0/*.amo
		rm -f /data/crop0/ingredient0/*.ZSM
		rm -f /data/crop0/ingredient0/*.zsm
		rm -f /data/crop0/ingredient0/lastknownmodels.conf
		rm -f /data/crop0/ingredient0/lastknownmodels_HMS.conf
		rm -f /data/crop0/ingredient0/lastknownmodels_MCS.conf
		rm -f /data/crop0/inactive/*
		
		if [ -n "$SW_VERSION" ]  &&  [ $SW_VERSION -gt 999 ]; then
			rm -f /data/crop0/ActivationList.DRM
		fi
	else
		echo "Post V41 Sensor Software Version '$SW_VERSION' detected."
		echo "--- leaving recent model files untouched..."
	fi

	echo "--- Installing new model files..."
	rm -f /data/HarvestLabTranslations.xml
	(tar -xvf czsensor.tar   -C /data/crop0/ingredient0   --wildcards *.AMO            2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar   -C /data/crop0/ingredient0   --wildcards *.zsm            2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar   -C /data                     HarvestLabTranslations.xml   2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	# JD-2791 [FR] adjust measurement value (dry/wet)
	(tar -xvf czsensor.tar   -C /data/crop0/ingredient0   DryWet.conf                  2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	
	echo invalMeasurementDataOnAmoFileDeact > /data/crop0/ingredient0/lastknownmodels.conf
	echo invalMeasurementDataOnAmoFileDeact > /data/crop0/ingredient0/lastknownmodels_HMS.conf
	echo invalMeasurementDataOnAmoFileDeact > /data/crop0/ingredient0/lastknownmodels_MCS.conf
	chown http:http /data/crop0/ingredient0/DryWet.conf
}


install_candriver()
{
	echo
	echo "installing can driver"
	echo "--------------------------------"
	echo
	(tar -xvf czsensor.tar    -C $KERNEL $DLLCAN 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
}

install_tools()
{
	echo
	echo "installing tools"
	echo "--------------------------------"
	echo
	(tar -xvf czsensor.tar    -C /bin unlink 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C /usr/bin mountusb.sh 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C /usr/bin amocp 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
	(tar -xvf czsensor.tar    -C /usr/bin amoeval 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
}

extract_apache()
{
	echo
	echo "installing APACHE webserver"
	echo "--------------------------------"
	echo
	cd /data
	(tar -xvz -f czsensor.tar -C /tmp $APACHEBIN 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
        (tar -x -f /tmp/$APACHEBIN -C /tmp 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
}

move_apache()
{
	mv /tmp/apache /usr/bin
	cd /usr/bin/apache
	ln -s -f /var/tmp logs
	rm /tmp/$APACHEBIN
}

finalize()
{
	echo
	echo "finalizing"
	echo "--------------------------------"
	echo
	echo "deleting czsensor.tar"
	rm /data/czsensor.tar
}

install_models()
{
	echo
	echo "copying models & filters"
	echo "-------------------------"
	
	cp $pathToSrc/models/* /data/crop0/ingredient0/
	ls -1 /data/crop0/ingredient0/
}

patch_inittab()
{
	sed "s/^::respawn:-\/bin\/sh/#::respawn:-\/bin\/sh/g" /etc/inittab -i
}




# start normal update procedure
############################################################

echo
echo "starting installation..."
echo 

cd /data

replace_tar
create_dirs_and_files
add_prerun_functionality
extract_tarball

extract_apache
move_apache
install_app_binaries
install_models
install_libraries
install_candriver
install_tools
patch_inittab
finalize

echo
echo "finished installation."
echo 
