#!/bin/sh

#/*************************************************************************
#**    Carl Zeiss MicroImaging GmbH
#**************************************************************************
#**
#**       File: update.sh
#**    Summary: update script for czsensor application binary
#**
#**    Version: 1.4
#**       Date: 10.10.2009
#**     Author: Johannes Paschold (j.paschold@zeiss.de)
#**     Author: Andreas Kirsch (a.kirsch@zeiss.de)
#**
#**        Log:    10/10/09 fixed JD-1646
#**
#** 06.04.2011
#**    - added variable "pathToSrc" with new path for releases after V41
#** 15.09.2011
#**    - new path for inactive calibrations (/data/crop0/inactive)
#**    - new path for persistent CSPRNG data (/data/csprng_log) and activation log (/data/csprng_log)
#**    - JD-2156: save sensor parameter file /usr/bin/czsensor/czsensor.config on firmware update
#**************************************************************************

export PATH=$PATH:/sbin
SOFTWAREVERSION=50  # Don't change anything else than the version number in this line! Preserve/Keep the space behind the number!!!
APPVERSION=0
ARCHIVENAME='czsensor.tar'

if [ "$1" = "--version" ]; then
   echo $VERSION
   exit
fi

echo
echo "save old configuration (feature request JD-2156)"
echo
# feature request JD-2156
# get sensor software version
VERSION=`cat /data/version`

# check the version of current sensor to change the update policy from pre-v41 to v41 update process
# pre-v41 uses USB for update
# v41     uses WebUI for update
if [ "$VERSION" \< "41" ];
then
    pathToSrc=/mtd/usbstick
else
    pathToSrc=/tmp
fi

# save configuration file
CONFDIR="/usr/bin/czsensor"
SAVEDIR="/data/saveConfigs"
mkdir -p $SAVEDIR
if [ -f "$SAVEDIR/czsensor.config_"$VERSION ]; then
    FILESCOUNT=`find "/data" -type f -name "czsensor.config_*" | wc -l`
    cp $CONFDIR/czsensor.config $SAVEDIR/czsensor.config'_'$VERSION'_'`expr $FILESCOUNT`
else
    cp $CONFDIR/czsensor.config $SAVEDIR/czsensor.config'_'$VERSION
fi


echo
echo "preparing update..."
echo

# ISSUE JD-1180
# replace busybox tar by gnu tar
if [ -f $pathToSrc/tar ]; then
    echo
    echo "replacing tar"
    echo "--------------------------------"
    echo
    rm /bin/tar
    cp $pathToSrc/tar /data/tar
    chmod 775 /data/tar
    ln -s /data/tar /bin/tar
    tar --version | grep tar
fi

#
# preparing directories
#
mkdir -p /data/crop0/ingredient0
mkdir -p /data/crop0/inactive
chown http:http /data/crop0/ingredient0
chown http:http /data/crop0/inactive
chmod 770 /data/crop0/ingredient0
chmod 770 /data/crop0/inactive

# create file and set rights/owner for CSPRNG
touch /data/csprng
touch /data/csprng_log
chown http:http /data/csprng
chown http:http /data/csprng_log
chmod 666 /data/csprng
chmod 666 /data/csprng_log

# create files for internal sensor status log
touch /data/errorStateLog
touch /data/errorStateLog.old
chmod 666 /data/errorStateLog
chmod 666 /data/errorStateLog.old


cd $pathToSrc

# add prerun functionality
#####################################

echo
echo "checking inittab"
echo "--------------------------------"
echo

AUTOUPDATECMD="::wait:/usr/bin/czsensor/prerun.sh"
if [ -z $(cat /etc/inittab | grep prerun.sh) ]; then
   echo "adding line $AUTOUPDATECMD"
   echo  $AUTOUPDATECMD >> /etc/inittab
else
   echo "prerun.sh already enabled"
fi
# fix JD-1452
if [ -f /usr/bin/czsensor/prerun.sh ]; then
   echo "prerun.sh script already present"   
else
   echo "extracting prerun.sh script..."
   (tar -xvf $ARCHIVENAME -C /usr/bin/czsensor prerun.sh 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
   echo "changing permissions..."
   chmod 755 /usr/bin/czsensor/prerun.sh
fi

# extract tarball
#####################################

echo
echo "extracting update files"
echo "--------------------------------"
echo

(tar -xvf $ARCHIVENAME -C /data czsensorupdate.tar.gz 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
(tar -xvf $ARCHIVENAME -C /data autoupdate.sh 2>&1 1>&3 | grep -v "future" 1>&2) 3>&1
chmod 755 /data/autoupdate.sh
sync
sleep 2

echo 
echo "rebooting to trigger update..."
echo 
/sbin/reboot
